<?php

namespace Drupal\video_embed_peertube\Plugin\video_embed_field\Provider;

use Drupal\video_embed_field\ProviderPluginBase;

/**
 * Peertube video provider.
 *
 * @VideoEmbedProvider(
 *   id = "peertube",
 *   title = @Translation("Peertube")
 * )
 */
class Peertube extends ProviderPluginBase {

  /**
   * {@inheritdoc}
   */
  public function renderEmbedCode($width, $height, $autoplay, $title_format = NULL, $use_title_fallback = TRUE) {
    $input = $this->getInput();
    $domain = self::extractInstanceSchemeDomain($input);
    $embed_code = [
      '#type' => 'html_tag',
      '#tag' => 'iframe',
      '#attributes' => [
        'width' => $width,
        'height' => $height,
        'frameborder' => '0',
        'allowfullscreen' => 'allowfullscreen',
        'src' => sprintf($domain . '/videos/embed/%s?autoplay=%d', $this->getVideoId(), $autoplay),
      ],
    ];
    // We add a title attribute to the iframe tag for better accessibility.
    // This feature is new in Video Embed Field 3.0.x.
    // @see https://dequeuniversity.com/checklists/web/iframes
    $title = $this->getName($title_format, $use_title_fallback);
    if (isset($title)) {
      $embed_code['#attributes']['title'] = $title;
    }
    return $embed_code;
  }

  /**
   * Get the PeerTube video data using API.
   *
   * @return mixed
   *   Data from the PeerTube api endpoint.
   */
  protected function videoData(): mixed {
    $input = $this->getInput();
    if (self::isAllowedInstance($input)) {
      $domain = self::extractInstanceSchemeDomain($input);
      $peertube_video_info = sprintf($domain . '/api/v1/videos/%s', $this->getVideoId());
      $json = file_get_contents($peertube_video_info);
      return json_decode($json);
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getName($title_format = NULL, $use_title_fallback = TRUE) {
    return $this->formatTitle(
      $this->videoData()?->name ?? NULL,
      $title_format,
      $use_title_fallback
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getRemoteThumbnailUrl() {
    $input = $this->getInput();
    $domain = self::extractInstanceSchemeDomain($input);
    return sprintf($domain . '/static/previews/%s.jpg', $this->getVideoId());
  }

  /**
   * {@inheritdoc}
   */
  public static function getIdFromInput($input) {
    static $ids_cache = [];
    if (!isset($ids_cache[$input])) {
      $matches = NULL;
      if (self::isAllowedInstance($input) && self::isPeerTubeInstance($input)) {
        $domain = self::extractInstanceDomain($input);
        // Extract the video ID.
        preg_match('/^https?:\/\/(?:www\.)?' . $domain . '\/(?:w|videos\/(?:watch|embed))\/(?<id>[0-9A-Za-z_-]+)/', $input, $matches);
      }
      $ids_cache[$input] = $matches['id'] ?? FALSE;
    }
    return $ids_cache[$input];
  }

  /**
   * Function to extract the domain name.
   *
   * @param string $input
   *   The video URL.
   *
   * @return string
   *   The domain name.
   */
  public static function extractInstanceDomain($input) {
    $info = parse_url($input);
    return !empty($info['host']) ? $info['host'] : '';
  }

  /**
   * Function to extract the domain name with the scheme.
   *
   * @param string $input
   *   The video URL.
   *
   * @return string
   *   The domain name with the scheme.
   */
  public static function extractInstanceSchemeDomain($input) {
    $info = parse_url($input);
    return !empty($info['host'] && !empty($info['scheme'])) ? $info['scheme'] . '://' . $info['host'] : '';
  }

  /**
   * Function to determine if the website is a Peertube instance.
   *
   * @param string $input
   *   The video URL.
   *
   * @return bool
   *   Is a Peertube instance.
   */
  public static function isPeerTubeInstance($input) {
    // Do a call on the Peertube API to know if it is a Peertube instance.
    $domain = self::extractInstanceSchemeDomain($input);
    $peertube_url_info = $domain . '/api/v1/config';
    $json = file_get_contents($peertube_url_info);
    $obj = json_decode($json);
    return !empty($obj) && !empty($obj->instance);
  }

  /**
   * Check that the Peertube instance is allowed in configuration.
   *
   * @param string $input
   *   The video url to validate.
   *
   * @return string|bool
   *   The host name, if allowed, FALSE otherwise
   *
   * @see https://www.drupal.org/project/video_embed_peertube/issues/3497983
   */
  public static function isAllowedInstance(string $input) {
    $allowed_instances = self::getAllowedInstances();
    if (empty($allowed_instances)) {
      // If no instances are configured, we allow all for BC.
      return TRUE;
    }
    $domain = self::extractInstanceSchemeDomain($input);
    return in_array($domain, $allowed_instances);
  }

  /**
   * Retrieves the list of allowed PeerTube instances.
   *
   * @return array
   *   An array of allowed PeerTube instance URLs.
   */
  protected static function getAllowedInstances() {
    static $allowed_instances = NULL;
    if (is_null($allowed_instances)) {
      $config = \Drupal::config('video_embed_peertube.settings');
      $allowed_instances_text = $config->get('allowed_instances') ?? '';
      $allowed_instances = preg_split('/\r\n|\r|\n/', $allowed_instances_text, -1, PREG_SPLIT_NO_EMPTY);
    }
    return $allowed_instances;
  }

}
